# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import model_template_list_params, model_template_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.model_template import ModelTemplate
from ..types.parameter_schema_param import ParameterSchemaParam
from ..types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["ModelTemplatesResource", "AsyncModelTemplatesResource"]


class ModelTemplatesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ModelTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ModelTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ModelTemplatesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        endpoint_type: Literal["SYNC", "ASYNC", "STREAMING", "BATCH"],
        model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"],
        name: str,
        vendor_configuration: model_template_create_params.VendorConfiguration,
        endpoint_protocol: Literal["SGP", "COHERE", "VLLM"] | NotGiven = NOT_GIVEN,
        model_creation_parameters_schema: ParameterSchemaParam | NotGiven = NOT_GIVEN,
        model_request_parameters_schema: ParameterSchemaParam | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelTemplate:
        """
        ### Description

        Creates a model template.

        ### Details

        Model templates serve 2 purposes. First, they provide common scaffolding that is
        static across multiple models. Second, they expose several variables that can be
        injected at model creation time to customize the model.

        For example, a model template can define a docker image that contains code to
        run a HuggingFace or SentenceTransformers model. This docker image code also
        accepts environment variables that can be set to swap out the model weights or
        model name.

        Two of the most important fields required to create a model template are the
        `model_creation_parameters_schema` and `model_request_parameters_schema` fields.

        The `model_creation_parameters_schema` field defines the schema for parameters
        that can be injected at model creation time. For example, if the schema contains
        a `model_weights_uri` field, which expects a string, when a model is created
        from this template, the user can provide a URI to a model weights file that can
        be used to swap out the model weights used by the model.

        The `model_request_parameters_schema` field defines the schema for parameters
        that can be injected by an end user at model execution time. For example, if the
        schema contains a `model_request_parameters` field, which expects a string, when
        a model is executed, the user can provide a string that will be passed to the
        model for inference.

        Args:
          account_id: The ID of the account that owns the given entity.

          endpoint_type: An enum representing the different types of model endpoint types supported.

              Attributes: SYNC: Denotes that the model endpoint type is sync. ASYNC: Denotes
              that the model endpoint type is async. STREAMING: Denotes that the model
              endpoint type is streaming. BATCH: Denotes that the model endpoint type is
              batch.

          model_type: An enum representing the different types of models supported.

              Attributes: COMPLETION: Denotes that the model type is completion.
              CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
              that the model type is agent. EMBEDDING: Denotes that the model type is
              embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
              that the model type is generic.

          vendor_configuration: Configuration for launching a model using the Launch service which is an
              internal and self-hosted service developed by Scale that deploys models on
              Kubernetes.

              Attributes: vendor: The vendor of the model template bundle_config: The bundle
              configuration of the model template endpoint_config: The endpoint configuration
              of the model template

          endpoint_protocol: An enumeration.

          model_creation_parameters_schema: The field names and types of available parameter fields which may be specified
              during model creation

          model_request_parameters_schema: The field names and types of available parameter fields which may be specified
              in a model execution API's `model_request_parameters` field.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/model-templates",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "endpoint_type": endpoint_type,
                    "model_type": model_type,
                    "name": name,
                    "vendor_configuration": vendor_configuration,
                    "endpoint_protocol": endpoint_protocol,
                    "model_creation_parameters_schema": model_creation_parameters_schema,
                    "model_request_parameters_schema": model_request_parameters_schema,
                },
                model_template_create_params.ModelTemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelTemplate,
        )

    def retrieve(
        self,
        model_template_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelTemplate:
        """
        ### Description

        Gets the details of a model template

        ### Details

        This API can be used to get information about a single model template by ID. To
        use this API, pass in the `id` that was returned from your Create Model Template
        API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_template_id:
            raise ValueError(f"Expected a non-empty value for `model_template_id` but received {model_template_id!r}")
        return self._get(
            f"/v4/model-templates/{model_template_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelTemplate,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[ModelTemplate]:
        """
        ### Description

        Lists all model templates accessible to the user.

        ### Details

        This API can be used to list model templates. If a user has access to multiple
        accounts, all model templates from all accounts the user is associated with will
        be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/model-templates",
            page=SyncPageResponse[ModelTemplate],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    model_template_list_params.ModelTemplateListParams,
                ),
            ),
            model=ModelTemplate,
        )

    def delete(
        self,
        model_template_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a model template

        ### Details

        This API can be used to delete a model template by ID. To use this API, pass in
        the `id` that was returned from your Create Model Template API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_template_id:
            raise ValueError(f"Expected a non-empty value for `model_template_id` but received {model_template_id!r}")
        return self._delete(
            f"/v4/model-templates/{model_template_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncModelTemplatesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncModelTemplatesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncModelTemplatesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelTemplatesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncModelTemplatesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        endpoint_type: Literal["SYNC", "ASYNC", "STREAMING", "BATCH"],
        model_type: Literal["COMPLETION", "CHAT_COMPLETION", "AGENT", "EMBEDDING", "RERANKING", "GENERIC"],
        name: str,
        vendor_configuration: model_template_create_params.VendorConfiguration,
        endpoint_protocol: Literal["SGP", "COHERE", "VLLM"] | NotGiven = NOT_GIVEN,
        model_creation_parameters_schema: ParameterSchemaParam | NotGiven = NOT_GIVEN,
        model_request_parameters_schema: ParameterSchemaParam | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelTemplate:
        """
        ### Description

        Creates a model template.

        ### Details

        Model templates serve 2 purposes. First, they provide common scaffolding that is
        static across multiple models. Second, they expose several variables that can be
        injected at model creation time to customize the model.

        For example, a model template can define a docker image that contains code to
        run a HuggingFace or SentenceTransformers model. This docker image code also
        accepts environment variables that can be set to swap out the model weights or
        model name.

        Two of the most important fields required to create a model template are the
        `model_creation_parameters_schema` and `model_request_parameters_schema` fields.

        The `model_creation_parameters_schema` field defines the schema for parameters
        that can be injected at model creation time. For example, if the schema contains
        a `model_weights_uri` field, which expects a string, when a model is created
        from this template, the user can provide a URI to a model weights file that can
        be used to swap out the model weights used by the model.

        The `model_request_parameters_schema` field defines the schema for parameters
        that can be injected by an end user at model execution time. For example, if the
        schema contains a `model_request_parameters` field, which expects a string, when
        a model is executed, the user can provide a string that will be passed to the
        model for inference.

        Args:
          account_id: The ID of the account that owns the given entity.

          endpoint_type: An enum representing the different types of model endpoint types supported.

              Attributes: SYNC: Denotes that the model endpoint type is sync. ASYNC: Denotes
              that the model endpoint type is async. STREAMING: Denotes that the model
              endpoint type is streaming. BATCH: Denotes that the model endpoint type is
              batch.

          model_type: An enum representing the different types of models supported.

              Attributes: COMPLETION: Denotes that the model type is completion.
              CHAT_COMPLETION: Denotes that the model type is chat completion. AGENT: Denotes
              that the model type is agent. EMBEDDING: Denotes that the model type is
              embedding. RERANKING: Denotes that the model type is reranking. GENERIC: Denotes
              that the model type is generic.

          vendor_configuration: Configuration for launching a model using the Launch service which is an
              internal and self-hosted service developed by Scale that deploys models on
              Kubernetes.

              Attributes: vendor: The vendor of the model template bundle_config: The bundle
              configuration of the model template endpoint_config: The endpoint configuration
              of the model template

          endpoint_protocol: An enumeration.

          model_creation_parameters_schema: The field names and types of available parameter fields which may be specified
              during model creation

          model_request_parameters_schema: The field names and types of available parameter fields which may be specified
              in a model execution API's `model_request_parameters` field.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/model-templates",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "endpoint_type": endpoint_type,
                    "model_type": model_type,
                    "name": name,
                    "vendor_configuration": vendor_configuration,
                    "endpoint_protocol": endpoint_protocol,
                    "model_creation_parameters_schema": model_creation_parameters_schema,
                    "model_request_parameters_schema": model_request_parameters_schema,
                },
                model_template_create_params.ModelTemplateCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelTemplate,
        )

    async def retrieve(
        self,
        model_template_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelTemplate:
        """
        ### Description

        Gets the details of a model template

        ### Details

        This API can be used to get information about a single model template by ID. To
        use this API, pass in the `id` that was returned from your Create Model Template
        API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_template_id:
            raise ValueError(f"Expected a non-empty value for `model_template_id` but received {model_template_id!r}")
        return await self._get(
            f"/v4/model-templates/{model_template_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelTemplate,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ModelTemplate, AsyncPageResponse[ModelTemplate]]:
        """
        ### Description

        Lists all model templates accessible to the user.

        ### Details

        This API can be used to list model templates. If a user has access to multiple
        accounts, all model templates from all accounts the user is associated with will
        be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/model-templates",
            page=AsyncPageResponse[ModelTemplate],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    model_template_list_params.ModelTemplateListParams,
                ),
            ),
            model=ModelTemplate,
        )

    async def delete(
        self,
        model_template_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a model template

        ### Details

        This API can be used to delete a model template by ID. To use this API, pass in
        the `id` that was returned from your Create Model Template API call as a path
        parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not model_template_id:
            raise ValueError(f"Expected a non-empty value for `model_template_id` but received {model_template_id!r}")
        return await self._delete(
            f"/v4/model-templates/{model_template_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class ModelTemplatesResourceWithRawResponse:
    def __init__(self, model_templates: ModelTemplatesResource) -> None:
        self._model_templates = model_templates

        self.create = to_raw_response_wrapper(
            model_templates.create,
        )
        self.retrieve = to_raw_response_wrapper(
            model_templates.retrieve,
        )
        self.list = to_raw_response_wrapper(
            model_templates.list,
        )
        self.delete = to_raw_response_wrapper(
            model_templates.delete,
        )


class AsyncModelTemplatesResourceWithRawResponse:
    def __init__(self, model_templates: AsyncModelTemplatesResource) -> None:
        self._model_templates = model_templates

        self.create = async_to_raw_response_wrapper(
            model_templates.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            model_templates.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            model_templates.list,
        )
        self.delete = async_to_raw_response_wrapper(
            model_templates.delete,
        )


class ModelTemplatesResourceWithStreamingResponse:
    def __init__(self, model_templates: ModelTemplatesResource) -> None:
        self._model_templates = model_templates

        self.create = to_streamed_response_wrapper(
            model_templates.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            model_templates.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            model_templates.list,
        )
        self.delete = to_streamed_response_wrapper(
            model_templates.delete,
        )


class AsyncModelTemplatesResourceWithStreamingResponse:
    def __init__(self, model_templates: AsyncModelTemplatesResource) -> None:
        self._model_templates = model_templates

        self.create = async_to_streamed_response_wrapper(
            model_templates.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            model_templates.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            model_templates.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            model_templates.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .scalar import (
    ScalarResource,
    AsyncScalarResource,
    ScalarResourceWithRawResponse,
    AsyncScalarResourceWithRawResponse,
    ScalarResourceWithStreamingResponse,
    AsyncScalarResourceWithStreamingResponse,
)
from .metrics import (
    MetricsResource,
    AsyncMetricsResource,
    MetricsResourceWithRawResponse,
    AsyncMetricsResourceWithRawResponse,
    MetricsResourceWithStreamingResponse,
    AsyncMetricsResourceWithStreamingResponse,
)
from .timeseries import (
    TimeseriesResource,
    AsyncTimeseriesResource,
    TimeseriesResourceWithRawResponse,
    AsyncTimeseriesResourceWithRawResponse,
    TimeseriesResourceWithStreamingResponse,
    AsyncTimeseriesResourceWithStreamingResponse,
)

__all__ = [
    "ScalarResource",
    "AsyncScalarResource",
    "ScalarResourceWithRawResponse",
    "AsyncScalarResourceWithRawResponse",
    "ScalarResourceWithStreamingResponse",
    "AsyncScalarResourceWithStreamingResponse",
    "TimeseriesResource",
    "AsyncTimeseriesResource",
    "TimeseriesResourceWithRawResponse",
    "AsyncTimeseriesResourceWithRawResponse",
    "TimeseriesResourceWithStreamingResponse",
    "AsyncTimeseriesResourceWithStreamingResponse",
    "MetricsResource",
    "AsyncMetricsResource",
    "MetricsResourceWithRawResponse",
    "AsyncMetricsResourceWithRawResponse",
    "MetricsResourceWithStreamingResponse",
    "AsyncMetricsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .application_test_case_output import ApplicationTestCaseOutput

__all__ = ["ApplicationTestCaseOutputBatchResponse"]

ApplicationTestCaseOutputBatchResponse: TypeAlias = List[ApplicationTestCaseOutput]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.evaluation_datasets import (
    evaluation_dataset_version_list_params,
    evaluation_dataset_version_create_params,
)
from ...types.evaluation_datasets.evaluation_dataset_version import EvaluationDatasetVersion

__all__ = ["EvaluationDatasetVersionsResource", "AsyncEvaluationDatasetVersionsResource"]


class EvaluationDatasetVersionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvaluationDatasetVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationDatasetVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationDatasetVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return EvaluationDatasetVersionsResourceWithStreamingResponse(self)

    def create(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetVersion:
        """
        ### Description

        Creates a evaluation dataset version

        ### Details

        This API can be used to create a evaluation dataset version. To use this API,
        review the request schema and pass in all fields that are required to create a
        evaluation dataset version.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions",
            body=maybe_transform(
                {"account_id": account_id},
                evaluation_dataset_version_create_params.EvaluationDatasetVersionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetVersion,
        )

    def retrieve(
        self,
        evaluation_dataset_version_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetVersion:
        """
        ### Description

        Gets the details of a evaluation dataset version

        ### Details

        This API can be used to get information about a single evaluation dataset
        version by ID. To use this API, pass in the `id` that was returned from your
        Create Evaluation Dataset Version API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not evaluation_dataset_version_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_version_id` but received {evaluation_dataset_version_id!r}"
            )
        return self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions/{evaluation_dataset_version_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetVersion,
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[EvaluationDatasetVersion]:
        """
        ### Description

        Lists all evaluation dataset versions accessible to the user.

        ### Details

        This API can be used to list evaluation dataset versions. If a user has access
        to multiple accounts, all evaluation dataset versions from all accounts the user
        is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions",
            page=SyncPageResponse[EvaluationDatasetVersion],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    evaluation_dataset_version_list_params.EvaluationDatasetVersionListParams,
                ),
            ),
            model=EvaluationDatasetVersion,
        )


class AsyncEvaluationDatasetVersionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvaluationDatasetVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationDatasetVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationDatasetVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncEvaluationDatasetVersionsResourceWithStreamingResponse(self)

    async def create(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetVersion:
        """
        ### Description

        Creates a evaluation dataset version

        ### Details

        This API can be used to create a evaluation dataset version. To use this API,
        review the request schema and pass in all fields that are required to create a
        evaluation dataset version.

        Args:
          account_id: The ID of the account that owns the given entity.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return await self._post(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions",
            body=await async_maybe_transform(
                {"account_id": account_id},
                evaluation_dataset_version_create_params.EvaluationDatasetVersionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetVersion,
        )

    async def retrieve(
        self,
        evaluation_dataset_version_id: str,
        *,
        evaluation_dataset_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationDatasetVersion:
        """
        ### Description

        Gets the details of a evaluation dataset version

        ### Details

        This API can be used to get information about a single evaluation dataset
        version by ID. To use this API, pass in the `id` that was returned from your
        Create Evaluation Dataset Version API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        if not evaluation_dataset_version_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_version_id` but received {evaluation_dataset_version_id!r}"
            )
        return await self._get(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions/{evaluation_dataset_version_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluationDatasetVersion,
        )

    def list(
        self,
        evaluation_dataset_id: str,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[EvaluationDatasetVersion, AsyncPageResponse[EvaluationDatasetVersion]]:
        """
        ### Description

        Lists all evaluation dataset versions accessible to the user.

        ### Details

        This API can be used to list evaluation dataset versions. If a user has access
        to multiple accounts, all evaluation dataset versions from all accounts the user
        is associated with will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `evaluation_dataset_id` but received {evaluation_dataset_id!r}"
            )
        return self._get_api_list(
            f"/v4/evaluation-datasets/{evaluation_dataset_id}/evaluation-dataset-versions",
            page=AsyncPageResponse[EvaluationDatasetVersion],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                    },
                    evaluation_dataset_version_list_params.EvaluationDatasetVersionListParams,
                ),
            ),
            model=EvaluationDatasetVersion,
        )


class EvaluationDatasetVersionsResourceWithRawResponse:
    def __init__(self, evaluation_dataset_versions: EvaluationDatasetVersionsResource) -> None:
        self._evaluation_dataset_versions = evaluation_dataset_versions

        self.create = to_raw_response_wrapper(
            evaluation_dataset_versions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            evaluation_dataset_versions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            evaluation_dataset_versions.list,
        )


class AsyncEvaluationDatasetVersionsResourceWithRawResponse:
    def __init__(self, evaluation_dataset_versions: AsyncEvaluationDatasetVersionsResource) -> None:
        self._evaluation_dataset_versions = evaluation_dataset_versions

        self.create = async_to_raw_response_wrapper(
            evaluation_dataset_versions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            evaluation_dataset_versions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            evaluation_dataset_versions.list,
        )


class EvaluationDatasetVersionsResourceWithStreamingResponse:
    def __init__(self, evaluation_dataset_versions: EvaluationDatasetVersionsResource) -> None:
        self._evaluation_dataset_versions = evaluation_dataset_versions

        self.create = to_streamed_response_wrapper(
            evaluation_dataset_versions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            evaluation_dataset_versions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            evaluation_dataset_versions.list,
        )


class AsyncEvaluationDatasetVersionsResourceWithStreamingResponse:
    def __init__(self, evaluation_dataset_versions: AsyncEvaluationDatasetVersionsResource) -> None:
        self._evaluation_dataset_versions = evaluation_dataset_versions

        self.create = async_to_streamed_response_wrapper(
            evaluation_dataset_versions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            evaluation_dataset_versions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            evaluation_dataset_versions.list,
        )



from typing import Any, Dict, List, Union, Optional

from scale_gp import BaseModel
from scale_gp.types.evaluation_datasets import FlexibleMessage


class MultiturnTestCaseSchema(BaseModel):
    messages: FlexibleMessage
    turns: Optional[List[int]]
    expected_messages: Optional[FlexibleMessage]
    other_inputs: Optional[Union[str, float, Dict[str, Any]]] = None
    other_expected: Optional[Union[str, float, Dict[str, Any]]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel
from .question import Question

__all__ = ["PaginatedQuestionSets", "Item"]


class Item(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    name: str

    instructions: Optional[str] = None
    """Instructions to answer questions"""

    questions: Optional[List[Question]] = None


class PaginatedQuestionSets(BaseModel):
    current_page: int
    """The current page number."""

    items: List[Item]
    """The data returned for the current page."""

    items_per_page: int
    """The number of items per page."""

    total_item_count: int
    """The total number of items of the query"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["EvaluationDatasetVersionCreateParams"]


class EvaluationDatasetVersionCreateParams(TypedDict, total=False):
    account_id: str
    """The ID of the account that owns the given entity."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["EvaluationDataset"]


class EvaluationDataset(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    draft: bool
    """Boolean to check whether or not the evaluation dataset is in draft mode"""

    name: str
    """The name of the dataset"""

    schema_type: Literal["GENERATION", "FLEXIBLE"]
    """An enumeration."""

    updated_at: datetime
    """The date and time when the entity was last updated in ISO format."""

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    evaluation_dataset_metadata: Optional[object] = None
    """Metadata about the Evaluation Dataset that it is associated with"""

    knowledge_base_id: Optional[str] = None
    """ID of the knowledge base that the evaluation dataset is associated with."""

    out_of_date: Optional[bool] = None
    """
    Boolean to check whether or not the knowledge base has been uploaded to since
    publication of the dataset.
    """

    vendor: Optional[Literal["scale"]] = None
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ..._models import BaseModel

__all__ = ["DeleteKnowledgeBaseDataSourceConnectionResponse"]


class DeleteKnowledgeBaseDataSourceConnectionResponse(BaseModel):
    artifacts_to_delete: Dict[str, int]

    deletion_workflow_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConfluenceDataSourceConfigParam"]


class ConfluenceDataSourceConfigParam(TypedDict, total=False):
    source: Required[Literal["Confluence"]]

    space_key: Required[str]
    """Confluence space key to retrieve contents from.

    See https://support.atlassian.com/confluence-cloud/docs/choose-a-space-key
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)
from .usage_statistics import (
    UsageStatisticsResource,
    AsyncUsageStatisticsResource,
    UsageStatisticsResourceWithRawResponse,
    AsyncUsageStatisticsResourceWithRawResponse,
    UsageStatisticsResourceWithStreamingResponse,
    AsyncUsageStatisticsResourceWithStreamingResponse,
)

__all__ = [
    "UsageStatisticsResource",
    "AsyncUsageStatisticsResource",
    "UsageStatisticsResourceWithRawResponse",
    "AsyncUsageStatisticsResourceWithRawResponse",
    "UsageStatisticsResourceWithStreamingResponse",
    "AsyncUsageStatisticsResourceWithStreamingResponse",
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
]



import inspect
import logging
from typing import Any, Dict, List, Type, Union, Callable, Iterable, Optional
from typing_extensions import Self, TypedDict

from scale_gp.types.shared_params.chunk_extra_info_schema import Chunk, ChunkExtraInfoSchema
from scale_gp.types.shared_params.result_schema_generation import ResultSchemaGeneration
from scale_gp.types.shared_params.string_extra_info_schema import StringExtraInfoSchema

from .._client import SGPClient
from ..types.evaluation_datasets.test_case import TestCase
from ..types.application_test_case_output_batch_params import (
    Item as _ApplicationTestCaseOutputItem,
    ItemTraceSpan,
)

log: logging.Logger = logging.getLogger("scale_gp")


class ExternalApplicationOutputCompletion(TypedDict):
    generation_output: str
    metrics: Optional[Dict[str, float]]
    trace_spans: Optional[Iterable[ItemTraceSpan]]


class ExternalApplicationOutputContextChunks(ExternalApplicationOutputCompletion):
    chunks: List[Chunk]


class ExternalApplicationOutputContextString(ExternalApplicationOutputCompletion):
    info: str


ExternalApplicationOutput = Union[
    ExternalApplicationOutputCompletion, ExternalApplicationOutputContextChunks, ExternalApplicationOutputContextString
]

ExternalApplicationCallable = Union[
    Callable[[Any], ExternalApplicationOutput],
    Callable[[Any, TestCase], ExternalApplicationOutput],
]


class ExternalApplication:
    def __init__(self, client: SGPClient):
        self.client = client
        self._initialized = False
        self._output_model: Optional[Type[ExternalApplicationOutput]] = None

    def initialize(self, *, application_variant_id: str, application: ExternalApplicationCallable) -> Self:
        application_variant = self.client.application_variants.retrieve(application_variant_id)

        if application_variant.version != "OFFLINE":
            raise ValueError(f"Application variant {application_variant_id} is not an external application.")

        self.application_variant = application_variant

        def application_wrapper(input_data: Union[str, object], test_case: TestCase) -> ExternalApplicationOutput:
            args: List[Any] = [input_data]
            if len(inspect.signature(application).parameters) == 2:
                args.append(test_case)
            return application(*args)

        self.application = application_wrapper
        self._initialized = True
        return self

    def generate_outputs(self, *, evaluation_dataset_id: str, evaluation_dataset_version: int) -> None:
        self._check_initialized()

        test_cases = self._retrieve_test_cases_to_run(evaluation_dataset_id, evaluation_dataset_version)
        test_case_id_to_output: Dict[str, ExternalApplicationOutput] = {}

        for test_case in test_cases:
            input_ = test_case.test_case_data.input
            log.info(f'\nRunning test case {test_case.id}\nInput: "{input_}"')

            output = self.application(input_, test_case)
            log.info(f'\nApplication responded with:\n"{output["generation_output"]}"')

            test_case_id_to_output[test_case.id] = output

        if test_case_id_to_output:
            self._create_outputs(evaluation_dataset_version, test_case_id_to_output)

        log.info(
            f"Created {len(test_case_id_to_output)} outputs on evaluation dataset {evaluation_dataset_id} version {evaluation_dataset_version} for application variant {self.application_variant.id}"
        )

    def _retrieve_test_cases_to_run(self, evaluation_dataset_id: str, version: int) -> List[TestCase]:
        existing_outputs_response = self.client.application_test_case_outputs.list(
            application_variant_id=self.application_variant.id,
            evaluation_dataset_id=evaluation_dataset_id,
            evaluation_dataset_version_num=version,
            limit=10_000,
        )

        existing_outputs = {output.test_case_id for output in existing_outputs_response.items}

        test_case_history_response = self.client.evaluation_datasets.test_cases.history.list(
            str(version),
            evaluation_dataset_id=evaluation_dataset_id,
            limit=10_000,
        )

        return [test_case for test_case in test_case_history_response.items if test_case.id not in existing_outputs]

    def _create_outputs(
        self,
        evaluation_dataset_version: int,
        test_case_id_to_output: Dict[str, ExternalApplicationOutput],
    ) -> None:
        items: List[_ApplicationTestCaseOutputItem] = []

        def get_output_dict(output: ExternalApplicationOutput) -> ResultSchemaGeneration:
            if "chunks" in output:
                return ResultSchemaGeneration(
                    generation_output=output["generation_output"],
                    generation_extra_info=ChunkExtraInfoSchema(
                        schema_type="CHUNKS",
                        chunks=output["chunks"],
                    ),
                )

            elif "info" in output:
                return ResultSchemaGeneration(
                    generation_output=output["generation_output"],
                    generation_extra_info=StringExtraInfoSchema(
                        schema_type="STRING",
                        info=output["info"],
                    ),
                )
            else:
                return ResultSchemaGeneration(
                    generation_output=output["generation_output"],
                )

        for test_case_id, output in test_case_id_to_output.items():
            item: _ApplicationTestCaseOutputItem = {
                "account_id": self.application_variant.account_id,
                "application_variant_id": self.application_variant.id,
                "evaluation_dataset_version_num": evaluation_dataset_version,
                "output": get_output_dict(output),
                "test_case_id": test_case_id,
            }
            if "metrics" in output and output["metrics"]:
                item["metrics"] = output["metrics"]
            if "trace_spans" in output and output["trace_spans"]:
                item["trace_spans"] = output["trace_spans"]
            items.append(item)

        self.client.application_test_case_outputs.batch(
            items=items,
        )

    def _check_initialized(self) -> None:
        if not self._initialized:
            raise ValueError(f"{self} is not initialized.")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .artifact import Artifact as Artifact
from .upload_schedule import UploadSchedule as UploadSchedule
from .chunk_list_params import ChunkListParams as ChunkListParams
from .upload_list_params import UploadListParams as UploadListParams
from .chunk_list_response import ChunkListResponse as ChunkListResponse
from .paginated_artifacts import PaginatedArtifacts as PaginatedArtifacts
from .artifact_list_params import ArtifactListParams as ArtifactListParams
from .upload_create_params import UploadCreateParams as UploadCreateParams
from .upload_list_response import UploadListResponse as UploadListResponse
from .async_job_list_params import AsyncJobListParams as AsyncJobListParams
from .knowledge_base_upload import KnowledgeBaseUpload as KnowledgeBaseUpload
from .artifact_list_response import ArtifactListResponse as ArtifactListResponse
from .upload_retrieve_params import UploadRetrieveParams as UploadRetrieveParams
from .async_job_list_response import AsyncJobListResponse as AsyncJobListResponse
from .artifact_retrieve_params import ArtifactRetrieveParams as ArtifactRetrieveParams
from .upload_schedule_with_views import UploadScheduleWithViews as UploadScheduleWithViews
from .upload_schedule_list_params import UploadScheduleListParams as UploadScheduleListParams
from .upload_schedule_create_params import UploadScheduleCreateParams as UploadScheduleCreateParams
from .upload_schedule_update_params import UploadScheduleUpdateParams as UploadScheduleUpdateParams
from .upload_schedule_retrieve_params import UploadScheduleRetrieveParams as UploadScheduleRetrieveParams
from .data_source_connection_delete_params import DataSourceConnectionDeleteParams as DataSourceConnectionDeleteParams
from .paginated_upload_schedule_with_views import PaginatedUploadScheduleWithViews as PaginatedUploadScheduleWithViews
from .cancel_knowledge_base_upload_response import (
    CancelKnowledgeBaseUploadResponse as CancelKnowledgeBaseUploadResponse,
)
from .create_knowledge_base_upload_response import (
    CreateKnowledgeBaseUploadResponse as CreateKnowledgeBaseUploadResponse,
)
from .delete_knowledge_base_data_source_connection_response import (
    DeleteKnowledgeBaseDataSourceConnectionResponse as DeleteKnowledgeBaseDataSourceConnectionResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AnnotationConfig", "Component"]


class Component(BaseModel):
    data_loc: List[str]

    label: Optional[str] = None


class AnnotationConfig(BaseModel):
    components: List[List[Component]]

    annotation_config_type: Optional[Literal["flexible", "summarization", "multiturn"]] = None
    """An enumeration."""

    direction: Optional[Literal["col", "row"]] = None
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ApplicationDeployment"]


class ApplicationDeployment(BaseModel):
    id: str

    account_id: str
    """The ID of the account that owns the given entity."""

    application_variant_id: str

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    endpoint: str

    is_active: bool

    name: str

    status: Literal["PENDING", "READY", "FAILED", "STOPPED"]
    """
    An enum representing the status of an application deployment. Attributes:
    PENDING: The deployment is pending. READY: The deployment is ready. FAILED: The
    deployment has failed. STOPPED: The deployment has stopped.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Mapping, cast
from typing_extensions import Literal

import httpx

from ...types import training_dataset_list_params, training_dataset_create_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from ..._utils import (
    extract_files,
    maybe_transform,
    deepcopy_minimal,
    async_maybe_transform,
)
from .contents import (
    ContentsResource,
    AsyncContentsResource,
    ContentsResourceWithRawResponse,
    AsyncContentsResourceWithRawResponse,
    ContentsResourceWithStreamingResponse,
    AsyncContentsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ..._base_client import AsyncPaginator, make_request_options
from ...types.training_dataset import TrainingDataset
from ...types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["TrainingDatasetsResource", "AsyncTrainingDatasetsResource"]


class TrainingDatasetsResource(SyncAPIResource):
    @cached_property
    def contents(self) -> ContentsResource:
        return ContentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TrainingDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return TrainingDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TrainingDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return TrainingDatasetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        file: FileTypes,
        name: str,
        schema_type: Literal["GENERATION", "RERANKING_QUESTIONS"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrainingDataset:
        """
        Create Training Dataset

        Args:
          account_id: The ID of the account that owns the given entity.

          file: The file to upload as the training dataset

          name: The name of the dataset

          schema_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "account_id": account_id,
                "file": file,
                "name": name,
                "schema_type": schema_type,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/v4/training-datasets",
            body=maybe_transform(body, training_dataset_create_params.TrainingDatasetCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrainingDataset,
        )

    def retrieve(
        self,
        training_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrainingDataset:
        """
        ### Description

        Gets the details of a training dataset

        ### Details

        This API can be used to get information about a single training dataset by ID.
        To use this API, pass in the `id` that was returned from your Create Training
        Dataset API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not training_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `training_dataset_id` but received {training_dataset_id!r}"
            )
        return self._get(
            f"/v4/training-datasets/{training_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrainingDataset,
        )

    def list(
        self,
        *,
        account_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[TrainingDataset]:
        """
        ### Description

        Lists all training datasets accessible to the user.

        ### Details

        This API can be used to list training datasets. If a user has access to multiple
        accounts, all training datasets from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/training-datasets",
            page=SyncPageResponse[TrainingDataset],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    training_dataset_list_params.TrainingDatasetListParams,
                ),
            ),
            model=TrainingDataset,
        )

    def delete(
        self,
        training_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a training dataset

        ### Details

        This API can be used to delete a training dataset by ID. To use this API, pass
        in the `id` that was returned from your Create Training Dataset API call as a
        path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not training_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `training_dataset_id` but received {training_dataset_id!r}"
            )
        return self._delete(
            f"/v4/training-datasets/{training_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class AsyncTrainingDatasetsResource(AsyncAPIResource):
    @cached_property
    def contents(self) -> AsyncContentsResource:
        return AsyncContentsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTrainingDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTrainingDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTrainingDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncTrainingDatasetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        file: FileTypes,
        name: str,
        schema_type: Literal["GENERATION", "RERANKING_QUESTIONS"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrainingDataset:
        """
        Create Training Dataset

        Args:
          account_id: The ID of the account that owns the given entity.

          file: The file to upload as the training dataset

          name: The name of the dataset

          schema_type: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "account_id": account_id,
                "file": file,
                "name": name,
                "schema_type": schema_type,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/v4/training-datasets",
            body=await async_maybe_transform(body, training_dataset_create_params.TrainingDatasetCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrainingDataset,
        )

    async def retrieve(
        self,
        training_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TrainingDataset:
        """
        ### Description

        Gets the details of a training dataset

        ### Details

        This API can be used to get information about a single training dataset by ID.
        To use this API, pass in the `id` that was returned from your Create Training
        Dataset API call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not training_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `training_dataset_id` but received {training_dataset_id!r}"
            )
        return await self._get(
            f"/v4/training-datasets/{training_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TrainingDataset,
        )

    def list(
        self,
        *,
        account_id: Union[int, str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[TrainingDataset, AsyncPageResponse[TrainingDataset]]:
        """
        ### Description

        Lists all training datasets accessible to the user.

        ### Details

        This API can be used to list training datasets. If a user has access to multiple
        accounts, all training datasets from all accounts the user is associated with
        will be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/training-datasets",
            page=AsyncPageResponse[TrainingDataset],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    training_dataset_list_params.TrainingDatasetListParams,
                ),
            ),
            model=TrainingDataset,
        )

    async def delete(
        self,
        training_dataset_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a training dataset

        ### Details

        This API can be used to delete a training dataset by ID. To use this API, pass
        in the `id` that was returned from your Create Training Dataset API call as a
        path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not training_dataset_id:
            raise ValueError(
                f"Expected a non-empty value for `training_dataset_id` but received {training_dataset_id!r}"
            )
        return await self._delete(
            f"/v4/training-datasets/{training_dataset_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )


class TrainingDatasetsResourceWithRawResponse:
    def __init__(self, training_datasets: TrainingDatasetsResource) -> None:
        self._training_datasets = training_datasets

        self.create = to_raw_response_wrapper(
            training_datasets.create,
        )
        self.retrieve = to_raw_response_wrapper(
            training_datasets.retrieve,
        )
        self.list = to_raw_response_wrapper(
            training_datasets.list,
        )
        self.delete = to_raw_response_wrapper(
            training_datasets.delete,
        )

    @cached_property
    def contents(self) -> ContentsResourceWithRawResponse:
        return ContentsResourceWithRawResponse(self._training_datasets.contents)


class AsyncTrainingDatasetsResourceWithRawResponse:
    def __init__(self, training_datasets: AsyncTrainingDatasetsResource) -> None:
        self._training_datasets = training_datasets

        self.create = async_to_raw_response_wrapper(
            training_datasets.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            training_datasets.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            training_datasets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            training_datasets.delete,
        )

    @cached_property
    def contents(self) -> AsyncContentsResourceWithRawResponse:
        return AsyncContentsResourceWithRawResponse(self._training_datasets.contents)


class TrainingDatasetsResourceWithStreamingResponse:
    def __init__(self, training_datasets: TrainingDatasetsResource) -> None:
        self._training_datasets = training_datasets

        self.create = to_streamed_response_wrapper(
            training_datasets.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            training_datasets.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            training_datasets.list,
        )
        self.delete = to_streamed_response_wrapper(
            training_datasets.delete,
        )

    @cached_property
    def contents(self) -> ContentsResourceWithStreamingResponse:
        return ContentsResourceWithStreamingResponse(self._training_datasets.contents)


class AsyncTrainingDatasetsResourceWithStreamingResponse:
    def __init__(self, training_datasets: AsyncTrainingDatasetsResource) -> None:
        self._training_datasets = training_datasets

        self.create = async_to_streamed_response_wrapper(
            training_datasets.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            training_datasets.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            training_datasets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            training_datasets.delete,
        )

    @cached_property
    def contents(self) -> AsyncContentsResourceWithStreamingResponse:
        return AsyncContentsResourceWithStreamingResponse(self._training_datasets.contents)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["StudioProjectUpdateParams"]


class StudioProjectUpdateParams(TypedDict, total=False):
    description: str
    """The description of the Studio Project"""

    name: str
    """The name of the Studio Project"""

    studio_api_key: str
    """Your API key for Studio, can be updated with the PATCH endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    ApplicationVariantListResponse,
    ApplicationVariantCreateResponse,
    ApplicationVariantRetrieveResponse,
)
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestApplicationVariants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: SGPClient) -> None:
        application_variant = client.application_variants.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                ],
            },
            name="name",
            version="V0",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: SGPClient) -> None:
        application_variant = client.application_variants.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                        "configuration": {"foo": {"value": {}}},
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                        "configuration": {"foo": {"value": {}}},
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                        "configuration": {"foo": {"value": {}}},
                    },
                ],
                "metadata": {},
            },
            name="name",
            version="V0",
            application_spec_id="application_spec_id",
            description="description",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                ],
            },
            name="name",
            version="V0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                ],
            },
            name="name",
            version="V0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: SGPClient) -> None:
        application_variant = client.application_variants.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: SGPClient) -> None:
        application_variant = client.application_variants.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
            application_spec_id="application_spec_id",
            configuration={},
            description="description",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_raw_response_create_overload_2(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_2(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_3(self, client: SGPClient) -> None:
        application_variant = client.application_variants.create(
            account_id="account_id",
            configuration={},
            name="name",
            version="OFFLINE",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_3(self, client: SGPClient) -> None:
        application_variant = client.application_variants.create(
            account_id="account_id",
            configuration={
                "metadata": {},
                "output_schema_type": "completion_only",
            },
            name="name",
            version="OFFLINE",
            application_spec_id="application_spec_id",
            description="description",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_raw_response_create_overload_3(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.create(
            account_id="account_id",
            configuration={},
            name="name",
            version="OFFLINE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_3(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.create(
            account_id="account_id",
            configuration={},
            name="name",
            version="OFFLINE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        application_variant = client.application_variants.retrieve(
            "application_variant_id",
        )
        assert_matches_type(ApplicationVariantRetrieveResponse, application_variant, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.retrieve(
            "application_variant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(ApplicationVariantRetrieveResponse, application_variant, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.retrieve(
            "application_variant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(ApplicationVariantRetrieveResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            client.application_variants.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        application_variant = client.application_variants.list()
        assert_matches_type(SyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        application_variant = client.application_variants.list(
            account_id="account_id",
            application_spec_id=0,
            limit=1,
            page=1,
        )
        assert_matches_type(SyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(SyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(
                SyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        application_variant = client.application_variants.delete(
            "application_variant_id",
        )
        assert_matches_type(GenericDeleteResponse, application_variant, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.delete(
            "application_variant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(GenericDeleteResponse, application_variant, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.delete(
            "application_variant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(GenericDeleteResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            client.application_variants.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_method_process(self, client: SGPClient) -> None:
        application_variant = client.application_variants.process(
            application_variant_id="application_variant_id",
            inputs={},
        )
        assert_matches_type(object, application_variant, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_method_process_with_all_params(self, client: SGPClient) -> None:
        application_variant = client.application_variants.process(
            application_variant_id="application_variant_id",
            inputs={},
            history=[
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
            ],
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
            stream=True,
        )
        assert_matches_type(object, application_variant, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_raw_response_process(self, client: SGPClient) -> None:
        response = client.application_variants.with_raw_response.process(
            application_variant_id="application_variant_id",
            inputs={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = response.parse()
        assert_matches_type(object, application_variant, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_streaming_response_process(self, client: SGPClient) -> None:
        with client.application_variants.with_streaming_response.process(
            application_variant_id="application_variant_id",
            inputs={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = response.parse()
            assert_matches_type(object, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    def test_path_params_process(self, client: SGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            client.application_variants.with_raw_response.process(
                application_variant_id="",
                inputs={},
            )


class TestAsyncApplicationVariants:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                ],
            },
            name="name",
            version="V0",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                        "configuration": {"foo": {"value": {}}},
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                        "configuration": {"foo": {"value": {}}},
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                        "configuration": {"foo": {"value": {}}},
                    },
                ],
                "metadata": {},
            },
            name="name",
            version="V0",
            application_spec_id="application_spec_id",
            description="description",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                ],
            },
            name="name",
            version="V0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.create(
            account_id="account_id",
            configuration={
                "edges": [
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                    {
                        "from_field": "from_field",
                        "from_node": "from_node",
                        "to_field": "to_field",
                        "to_node": "to_node",
                    },
                ],
                "nodes": [
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                    {
                        "id": "id",
                        "application_node_schema_id": "text_input_schema",
                    },
                ],
            },
            name="name",
            version="V0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
            application_spec_id="application_spec_id",
            configuration={},
            description="description",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.create(
            account_id="account_id",
            name="name",
            version="AGENTS_SERVICE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_3(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.create(
            account_id="account_id",
            configuration={},
            name="name",
            version="OFFLINE",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_3(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.create(
            account_id="account_id",
            configuration={
                "metadata": {},
                "output_schema_type": "completion_only",
            },
            name="name",
            version="OFFLINE",
            application_spec_id="application_spec_id",
            description="description",
        )
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_3(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.create(
            account_id="account_id",
            configuration={},
            name="name",
            version="OFFLINE",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_3(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.create(
            account_id="account_id",
            configuration={},
            name="name",
            version="OFFLINE",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(ApplicationVariantCreateResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.retrieve(
            "application_variant_id",
        )
        assert_matches_type(ApplicationVariantRetrieveResponse, application_variant, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.retrieve(
            "application_variant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(ApplicationVariantRetrieveResponse, application_variant, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.retrieve(
            "application_variant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(ApplicationVariantRetrieveResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            await async_client.application_variants.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.list()
        assert_matches_type(AsyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.list(
            account_id="account_id",
            application_spec_id=0,
            limit=1,
            page=1,
        )
        assert_matches_type(AsyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(AsyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(
                AsyncPageResponse[ApplicationVariantListResponse], application_variant, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.delete(
            "application_variant_id",
        )
        assert_matches_type(GenericDeleteResponse, application_variant, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.delete(
            "application_variant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(GenericDeleteResponse, application_variant, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.delete(
            "application_variant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(GenericDeleteResponse, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            await async_client.application_variants.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_method_process(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.process(
            application_variant_id="application_variant_id",
            inputs={},
        )
        assert_matches_type(object, application_variant, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_method_process_with_all_params(self, async_client: AsyncSGPClient) -> None:
        application_variant = await async_client.application_variants.process(
            application_variant_id="application_variant_id",
            inputs={},
            history=[
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
                {
                    "request": "request",
                    "response": "response",
                    "session_data": {},
                },
            ],
            overrides={
                "foo": {
                    "artifact_ids_filter": ["string", "string", "string"],
                    "artifact_name_regex": ["string", "string", "string"],
                    "type": "knowledge_base_schema",
                }
            },
            stream=True,
        )
        assert_matches_type(object, application_variant, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_raw_response_process(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.application_variants.with_raw_response.process(
            application_variant_id="application_variant_id",
            inputs={},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        application_variant = await response.parse()
        assert_matches_type(object, application_variant, path=["response"])

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_streaming_response_process(self, async_client: AsyncSGPClient) -> None:
        async with async_client.application_variants.with_streaming_response.process(
            application_variant_id="application_variant_id",
            inputs={},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            application_variant = await response.parse()
            assert_matches_type(object, application_variant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="additional_properties is an object")
    @parametrize
    async def test_path_params_process(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `application_variant_id` but received ''"
        ):
            await async_client.application_variants.with_raw_response.process(
                application_variant_id="",
                inputs={},
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..types import question_set_list_params, question_set_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.question_set import QuestionSet
from ..types.question_set_list_response import QuestionSetListResponse
from ..types.question_set_with_questions import QuestionSetWithQuestions

__all__ = ["QuestionSetsResource", "AsyncQuestionSetsResource"]


class QuestionSetsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QuestionSetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return QuestionSetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QuestionSetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return QuestionSetsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        name: str,
        question_ids: List[str],
        instructions: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuestionSet:
        """
        ### Description

        Creates a question set

        ### Details

        This API can be used to create a question set. To use this API, review the
        request schema and pass in all fields that are required to create a question
        set.

        Args:
          account_id: The ID of the account that owns the given entity.

          question_ids: IDs of questions in the question set

          instructions: Instructions to answer questions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/question-sets",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "question_ids": question_ids,
                    "instructions": instructions,
                },
                question_set_create_params.QuestionSetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuestionSet,
        )

    def retrieve(
        self,
        question_set_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuestionSetWithQuestions:
        """
        Get Question Set

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_set_id:
            raise ValueError(f"Expected a non-empty value for `question_set_id` but received {question_set_id!r}")
        return self._get(
            f"/v4/question-sets/{question_set_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuestionSetWithQuestions,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["Questions"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[QuestionSetListResponse]:
        """
        ### Description

        Lists all question sets accessible to the user.

        ### Details

        This API can be used to list question sets. If a user has access to multiple
        accounts, all question sets from all accounts the user is associated with will
        be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/question-sets",
            page=SyncPageResponse[QuestionSetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    question_set_list_params.QuestionSetListParams,
                ),
            ),
            model=QuestionSetListResponse,
        )


class AsyncQuestionSetsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQuestionSetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQuestionSetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQuestionSetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncQuestionSetsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        name: str,
        question_ids: List[str],
        instructions: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuestionSet:
        """
        ### Description

        Creates a question set

        ### Details

        This API can be used to create a question set. To use this API, review the
        request schema and pass in all fields that are required to create a question
        set.

        Args:
          account_id: The ID of the account that owns the given entity.

          question_ids: IDs of questions in the question set

          instructions: Instructions to answer questions

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/question-sets",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "name": name,
                    "question_ids": question_ids,
                    "instructions": instructions,
                },
                question_set_create_params.QuestionSetCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuestionSet,
        )

    async def retrieve(
        self,
        question_set_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuestionSetWithQuestions:
        """
        Get Question Set

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_set_id:
            raise ValueError(f"Expected a non-empty value for `question_set_id` but received {question_set_id!r}")
        return await self._get(
            f"/v4/question-sets/{question_set_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuestionSetWithQuestions,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        view: List[Literal["Questions"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[QuestionSetListResponse, AsyncPageResponse[QuestionSetListResponse]]:
        """
        ### Description

        Lists all question sets accessible to the user.

        ### Details

        This API can be used to list question sets. If a user has access to multiple
        accounts, all question sets from all accounts the user is associated with will
        be returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/question-sets",
            page=AsyncPageResponse[QuestionSetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                        "view": view,
                    },
                    question_set_list_params.QuestionSetListParams,
                ),
            ),
            model=QuestionSetListResponse,
        )


class QuestionSetsResourceWithRawResponse:
    def __init__(self, question_sets: QuestionSetsResource) -> None:
        self._question_sets = question_sets

        self.create = to_raw_response_wrapper(
            question_sets.create,
        )
        self.retrieve = to_raw_response_wrapper(
            question_sets.retrieve,
        )
        self.list = to_raw_response_wrapper(
            question_sets.list,
        )


class AsyncQuestionSetsResourceWithRawResponse:
    def __init__(self, question_sets: AsyncQuestionSetsResource) -> None:
        self._question_sets = question_sets

        self.create = async_to_raw_response_wrapper(
            question_sets.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            question_sets.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            question_sets.list,
        )


class QuestionSetsResourceWithStreamingResponse:
    def __init__(self, question_sets: QuestionSetsResource) -> None:
        self._question_sets = question_sets

        self.create = to_streamed_response_wrapper(
            question_sets.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            question_sets.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            question_sets.list,
        )


class AsyncQuestionSetsResourceWithStreamingResponse:
    def __init__(self, question_sets: AsyncQuestionSetsResource) -> None:
        self._question_sets = question_sets

        self.create = async_to_streamed_response_wrapper(
            question_sets.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            question_sets.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            question_sets.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ApplicationNodeParam", "Configuration"]


class Configuration(TypedDict, total=False):
    value: Required[object]


class ApplicationNodeParam(TypedDict, total=False):
    id: Required[str]

    application_node_schema_id: Required[
        Literal[
            "text_input_schema",
            "text_output_schema",
            "knowledge_base_schema",
            "reranker_schema",
            "prompt_engineering_schema",
            "completion_model_schema",
            "external_endpoint_schema",
            "document_input_schema",
            "map_reduce_schema",
            "document_search_schema",
            "document_prompt_schema",
        ]
    ]
    """An enumeration."""

    configuration: Dict[str, Configuration]
